// Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
// This source code is licensed under the Apache 2.0 license found
// in the LICENSE file in the root directory of this source tree.

// Code generated by entc, DO NOT EDIT.

package hook

import (
	"context"
	"fmt"

	"github.com/facebookincubator/ent/entc/integration/migrate/entv2"
)

// The CarFunc type is an adapter to allow the use of ordinary
// function as Car mutator.
type CarFunc func(context.Context, *entv2.CarMutation) (entv2.Value, error)

// Mutate calls f(ctx, m).
func (f CarFunc) Mutate(ctx context.Context, m entv2.Mutation) (entv2.Value, error) {
	mv, ok := m.(*entv2.CarMutation)
	if !ok {
		return nil, fmt.Errorf("unexpected mutation type %T. expect *entv2.CarMutation", m)
	}
	return f(ctx, mv)
}

// The GroupFunc type is an adapter to allow the use of ordinary
// function as Group mutator.
type GroupFunc func(context.Context, *entv2.GroupMutation) (entv2.Value, error)

// Mutate calls f(ctx, m).
func (f GroupFunc) Mutate(ctx context.Context, m entv2.Mutation) (entv2.Value, error) {
	mv, ok := m.(*entv2.GroupMutation)
	if !ok {
		return nil, fmt.Errorf("unexpected mutation type %T. expect *entv2.GroupMutation", m)
	}
	return f(ctx, mv)
}

// The PetFunc type is an adapter to allow the use of ordinary
// function as Pet mutator.
type PetFunc func(context.Context, *entv2.PetMutation) (entv2.Value, error)

// Mutate calls f(ctx, m).
func (f PetFunc) Mutate(ctx context.Context, m entv2.Mutation) (entv2.Value, error) {
	mv, ok := m.(*entv2.PetMutation)
	if !ok {
		return nil, fmt.Errorf("unexpected mutation type %T. expect *entv2.PetMutation", m)
	}
	return f(ctx, mv)
}

// The UserFunc type is an adapter to allow the use of ordinary
// function as User mutator.
type UserFunc func(context.Context, *entv2.UserMutation) (entv2.Value, error)

// Mutate calls f(ctx, m).
func (f UserFunc) Mutate(ctx context.Context, m entv2.Mutation) (entv2.Value, error) {
	mv, ok := m.(*entv2.UserMutation)
	if !ok {
		return nil, fmt.Errorf("unexpected mutation type %T. expect *entv2.UserMutation", m)
	}
	return f(ctx, mv)
}

// On executes the given hook only of the given operation.
//
//	hook.On(Log, entv2.Delete|entv2.Create)
//
func On(hk entv2.Hook, op entv2.Op) entv2.Hook {
	return func(next entv2.Mutator) entv2.Mutator {
		return entv2.MutateFunc(func(ctx context.Context, m entv2.Mutation) (entv2.Value, error) {
			if m.Op().Is(op) {
				return hk(next).Mutate(ctx, m)
			}
			return next.Mutate(ctx, m)
		})
	}
}

// Reject returns a hook that rejects all operations that match op.
//
//	func (T) Hooks() []entv2.Hook {
//		return []entv2.Hook{
//			Reject(entv2.Delete|entv2.Update),
//		}
//	}
//
func Reject(op entv2.Op) entv2.Hook {
	return func(next entv2.Mutator) entv2.Mutator {
		return entv2.MutateFunc(func(ctx context.Context, m entv2.Mutation) (entv2.Value, error) {
			if m.Op().Is(op) {
				return nil, fmt.Errorf("%s operation is not allowed", m.Op())
			}
			return next.Mutate(ctx, m)
		})
	}
}

// Chain acts as a list of hooks and is effectively immutable.
// Once created, it will always hold the same set of hooks in the same order.
type Chain struct {
	hooks []entv2.Hook
}

// NewChain creates a new chain of hooks.
func NewChain(hooks ...entv2.Hook) Chain {
	return Chain{append([]entv2.Hook(nil), hooks...)}
}

// Hook chains the list of hooks and returns the final hook.
func (c Chain) Hook() entv2.Hook {
	return func(mutator entv2.Mutator) entv2.Mutator {
		for i := len(c.hooks) - 1; i >= 0; i-- {
			mutator = c.hooks[i](mutator)
		}
		return mutator
	}
}

// Append extends a chain, adding the specified hook
// as the last ones in the mutation flow.
func (c Chain) Append(hooks ...entv2.Hook) Chain {
	newHooks := make([]entv2.Hook, 0, len(c.hooks)+len(hooks))
	newHooks = append(newHooks, c.hooks...)
	newHooks = append(newHooks, hooks...)
	return Chain{newHooks}
}

// Extend extends a chain, adding the specified chain
// as the last ones in the mutation flow.
func (c Chain) Extend(chain Chain) Chain {
	return c.Append(chain.hooks...)
}
